/* strings.h - string comparisons.
 *  Copyright (C) 2022 by Jed Marti
 *   This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see https://www.gnu.org/licenses/.
 */


#ifndef strings_h
#define strings_h

__bit strCaseEq(char __code *b);

__bit strEq(char *a, char *b);
char __xdata *strCpy(__xdata char *dest, char *src);

uint16_t strlenSTR(void);
uint16_t newStringSTR(void);

// String length in program space at offset PC.
uint16_t strlenI(void);

// Create a new string from the program I space at PC, result index returned.
uint16_t newStringI(void);

// Convert integer in IACC to a string.
char __xdata *int2string(char __xdata *s);

// Convert float to a string.
char __xdata *float2string(char __xdata *s);

#endif



// EOF
